#ifndef F_CPU
	#define F_CPU 12000000
#endif

#include <avr/interrupt.h>
#include "i2clcd.h"
#define uchar unsigned char
#define uint unsigned int

// Pin IR
#define	xRC5_IN		PIND
#define	xRC5		PD1							// IR input low active
#define RC5TIME 	1.778e-3					// 1.778msec
#define PULSE_MIN	(uchar)(F_CPU / 512 * RC5TIME * 0.4 + 0.5)
#define PULSE_1_2	(uchar)(F_CPU / 512 * RC5TIME * 0.8 + 0.5)
#define PULSE_MAX	(uchar)(F_CPU / 512 * RC5TIME * 1.2 + 0.5)
volatile uchar	rc5_bit;						// bit value
volatile uchar	rc5_time;						// count bit time
volatile uint	rc5_tmp;						// shift bits in
volatile uint	rc5_data;						// store result

//
// ISR fuer RC5-IR-Empfaenger
//
ISR(TIMER0_OVF_vect)
{
	uint tmp = rc5_tmp;							// for faster access
	TCNT0 = -2;									// 2 * 256 = 512 cycle
	if( ++rc5_time > PULSE_MAX ){				// count pulse time
		if( !(tmp & 0x4000) && tmp & 0x2000 )	// only if 14 bits received
			rc5_data = tmp;
		tmp = 0;
	}
	if( (rc5_bit ^ xRC5_IN) & 1<<xRC5 ){		// change detect
		rc5_bit = ~rc5_bit;						// 0x00 -> 0xFF -> 0x00
		if( rc5_time < PULSE_MIN )				// to short
			tmp = 0;
		if( !tmp || rc5_time > PULSE_1_2 ){		// start or long pulse time
			if( !(tmp & 0x4000) )				// not to many bits
				tmp <<= 1;						// shift
			if( !(rc5_bit & 1<<xRC5) )			// inverted bit
				tmp |= 1;						// insert new bit
			rc5_time = 0;						// count next pulse time
		}
	}
	rc5_tmp = tmp;
}

int main(void)
{
	uint temp_key;
	uint rc5_device;
	uint rc5_key_code;
	uint rc5_toggle_bit;
	char s[3];

	// Timer fuer RC5
	TCCR0 = 1<<CS02;							// divide by 256
	TIMSK = 1<<TOIE0;							// enable timer interrupt
    sei();	

	// TWI initialisieren
	i2c_init();

	// LCD initialisieren
	lcd_init();

	// LCD-Anzeige loeschen
	lcd_command(LCD_CLEAR);
	lcd_backlight(ON);

	// Text auf LCD ausgeben
	lcd_printlc(1,1,(unsigned char *)"-->BLIT-Board<--");
	lcd_printlc(2,1,(unsigned char *)"Uwe Berger; 2008");
	lcd_wait_ms(1000);

	lcd_command(LCD_CLEAR);
	lcd_printlc(1,1,(unsigned char *)" IR-RC5-Sniffer ");
	lcd_printlc(2,1,(unsigned char *)"press remote key");
	while (true){
		cli();               
		temp_key = rc5_data;			
		rc5_data = 0;
		sei();
		if (temp_key) {
			lcd_command(LCD_CLEAR);
			lcd_printlc(1,1,(unsigned char *)" IR-RC5-Sniffer ");
			rc5_device     = temp_key >> 6 & 0x1F;							// decice
			rc5_toggle_bit = temp_key >> 11 & 1;							// toggle bit
			rc5_key_code   = ((temp_key & 0x3F) | (~temp_key >> 7 & 0x40));	// key code
			itoa(rc5_device, s, 10);	
			lcd_printlc(2, 1, (unsigned char *)"D->");
			lcd_printlc(2, 4, (unsigned char *)s);						
			itoa(rc5_toggle_bit, s, 10);	
			lcd_printlc(2, 6, (unsigned char *)"T->");
			lcd_printlc(2, 9, (unsigned char *)s);						
			itoa(rc5_key_code, s, 10);	
			lcd_printlc(2, 11, (unsigned char *)"K->");
			lcd_printlc(2, 14, (unsigned char *)s);						
		}
	}					
}
